@@ -65,6 +65,8 @@ typedef enum { NO_HIT, LEFT_HIT, RIGHT_HIT, TOP_HIT, BOTTOM_HIT } hit;
 #define s_noeud sizeof(struct noeud)
 
 char *preptree(char *fname);
+void make_binary_or_unrooted(char *arbre);
+int make_binary(char *arbre, char *debut, char *fin, int go_down);
 void loadphylip(char *arbre);
 struct noeud *unrootedset(char *arbre, char *deb, char *fin, branche_noeud **p_int_br);
 char *nextpar(char *text, char *pospar);
@@ -129,18 +131,18 @@ extern double log_min_x, log_min_y, log_max_x, log_max_y, mini_br_length;
 
 char *preptree(char *fname)
 {
-int i, c, steparbre, maxarbre;
+int i, c, v, steparbre, maxarbre;
 FILE *njfile;
 char *arbre, *der_arbre, *finarbre, *tmp;
 
 if( (njfile=fopen(fname,"r")) == NULL )return ("Tree file not found.");
 /* recherche du debut de la description de l'arbre */
 do c=fgetc(njfile);
-while (c == '\n' || c == ' ' );
+while (c == '\n'  || c == '\r' || c == ' ' );
 /* for fastDNAml format, skip initial [comment] */
 if(c == '[') {
 	do c=fgetc(njfile); while (c != ']');
-	do c=fgetc(njfile); while (c == '\n' || c == ' ' );
+	do c=fgetc(njfile); while (c == '\n' || c == '\r' || c == ' ' );
 	}
 if ( c != '(') goto erreur;
 	/* lecture de l'arbre par paquets de steparbre caracteres*/
@@ -149,7 +151,7 @@ if ( c != '(') goto erreur;
 	arbre=check_alloc(maxarbre,1);
 	der_arbre = arbre+maxarbre;
 	*arbre=c;
-	notu=2; i=3;
+	notu=2; i=3; v = 0;
 	finarbre=arbre;
 	while( (c=fgetc(njfile)) != EOF && c != ';') {
 		if( /* c==' ' || */ c=='\n' || c=='\r') continue;
@@ -165,11 +167,16 @@ if ( c != '(') goto erreur;
 		*(finarbre)=c;
 		if(c==')')notu++;
 		if(c=='(')i++;
+		if(c == ',') v++;
 		}
 	*(finarbre+1)='\0';
 fclose(njfile);
 if(i!=notu)goto erreur;
-notu--;
+
+arbre = (char *)realloc(arbre, strlen(arbre) + 4 * v + 5 ); /* worst case add 4 chars for each , */
+make_binary_or_unrooted(arbre);
+/* notu--; */
+notu = v ; /* after this notu = number of OTUs - 1  */
 totbranches= -1;
 
 /* allocate all memory */
@@ -214,6 +221,86 @@ return ("Not a valid tree file: ");
 } /* end of preptree */
 
 
+void make_binary_or_unrooted(char *arbre)
+{
+char *finarbre, *deba, *debb, *debc;
+
+finarbre= nextpar(arbre,arbre);
+*(finarbre + 1) = 0;
+deba=arbre+1;
+debb=deba;
+while(*debb != ',') {
+	if(*debb == 0) { err_message("Incorrect tree file"); exit(1); }
+	if(*debb == '(')debb=nextpar(debb,debb);
+	debb++;
+	}
+debb++;
+debc=debb;
+while(*debc != ',' && debc<finarbre) {
+	if(*debc == '(')debc=nextpar(debc,debc);
+	debc++;
+	}
+if(*debc == ',') {
+/* the tree is unrooted <==> it has 3 subtrees or more at its bottommost level */
+	make_binary(arbre , debb, finarbre - 1, TRUE);
+	make_binary(arbre , deba, debb - 2, TRUE);
+	}
+else make_binary(arbre , deba, finarbre - 1, TRUE);
+return;
+}
+
+
+int make_binary(char *arbre, char *debut, char *fin, int go_down)
+{
+int virg, l, retval;
+char *p, *q;
+
+p = debut; virg = 0; retval = 0;
+while(p < fin) {
+	if(*p == ',') virg++;
+	else if(*p == '(')	{
+		q = nextpar(p,p);
+		if(go_down) {
+			l = make_binary(arbre, p+1, q-1, TRUE);
+			fin += l;
+			retval += l;
+			p = q + l;
+			}
+		else p = q;
+		}
+	p++;
+	}
+if(virg > 1) { /* multifurcation */
+	/* make sure tree has branch lengths */
+	if( memchr(debut, ':', fin - debut + 1) == NULL) {
+		err_message("Cannot process multibranched tree without branch lengths");
+		exit(1);
+		}
+	/* recherche de la 2eme virgule */
+	p = debut; l = 0;
+	while(TRUE) {
+		if(*p == ',') {
+			l++;
+			if(l == 2) break;
+			}
+		else if(*p == '(')	{
+			p = nextpar(p,p);
+			}
+		p++;
+		}
+	l = strlen(p);
+	memmove(p + 4, p, l + 1);
+	memmove(debut + 1, debut, p - debut);
+	*debut = '(';
+	memcpy(p + 1, "):0", 3);
+	fin += 4;
+	retval += 4;
+	if(virg > 2) retval += make_binary(arbre, debut, fin, FALSE);  
+	}
+return retval;
+}
+
+
 void loadphylip(char *arbre)
 {
 char *deba,*debb,*debc, *finarbre;
@@ -247,8 +334,9 @@ else	{
 /* the tree is rooted */
 	debc=finarbre+1;
 	rooted=1;
-/*!!!!!!!! notu computed before was 1 unit too high !!!!!!!!*/
-	notu--;
+/*
+	notu--;  //useless now
+*/
 	}
 num_noeud=notu;
 nextotu= -1;
@@ -316,7 +404,8 @@ if(*virg == ',') {
 if(p1 == NULL) return NULL;
 	p2=unrootedset(arbre,virg+1,fin,&int_br_d);
 if(p2 == NULL) return NULL;
-if(num_noeud + 1 >= 2*notu + 1) return NULL;
+if(num_noeud + 1 >= 2*notu + 1) 
+			return NULL;
 	p = *(tabtax+(++num_noeud));
 	p->v1=p1; p1->v3=p; p->l1=p1->l3;
 	if(int_br_g!=NULL) { int_br_g->bouta=p; int_br_g->boutb=p1; }
